% constructinverseir.m
%
% Function for constructing an inverse filter for the loudspeaker element
% used in the orchestra pit measurement series by Anders Lovstad.
%
% This version uses a bandpass-filter as target function.
% The inverse ir is flat +0.02 dB, - 0.25 dB for 442-22800 Hz.
%
% Anders Lovstad and Peter Svensson, 31 March 2008
% svensson@iet.ntnu.no

%-----------------------------------------------
% Parameters

Filepath = './Measurement files/Mic cal/';
samplenumbers = [150:250];
micnr=input('Type mid (1) or upper (2) mic: ');
%-----------------------------------------------
% Loading the impulse response into a vector irfree

nsamples = length(samplenumbers);
Filename = [Filepath,'ch',int2str(micnr),'_32av.wmb'];
eval(['[ir, fs, Format]=loadimp(Filename);']);
irfree = ir(samplenumbers).';

nfft = 8192;
tf_free = fft(irfree,nfft);

ninverseirlength = 600;


ir_bp = fir1(ninverseirlength,[250 23000]/(fs/2),blackmanharris(ninverseirlength+1));
tf_bp = fft(ir_bp,nfft);

tf_inv = tf_bp./tf_free;
ir_inv = real(ifft(tf_inv));
ir_inv = ir_inv(1:ninverseirlength);

fvec = fs/nfft*[0:nfft/2-1];

%-------------------------------------------------
% Test of what is produced

figure(5)
plot(irfree)
title('Original, measured direct sound IR')

figure(6)
semilogx(fvec,20*log(abs(tf_free(1:nfft/2))),fvec,20*log(abs(tf_inv(1:nfft/2))))
% axis([100 30000 -40 0]);
title('Original, measured direct sound IR, viewed as a TF, and target TF')
grid

figure(1)
plot(ir_bp)
title('BP filtered, equalized IR')

figure(2)
plot(ir_inv);
title('The Eq./inversion filter IR, after windowing')
axis([0 ninverseirlength -0.4 1.4]);

irtest = conv(ir_inv,irfree);
tftest = fft(irtest, nfft);
fvec = (fs/nfft)*[0:nfft/2-1];
figure(3)
plot(fvec,20*log(abs(tftest(1:nfft/2))))
axis([100 30000 -2 2]);
title('The BP filtered, equalized IR, viewed as a TF')
grid

figure(4)
plot(irh)
title('The window that is applied to the eq./inversion filter')


